package turgen.plugin.plturbo;

import turgen.plugin.*;
import turgen.generator.SignalGenerator;
import turgen.config.Configuration;
import turgen.*;
import java.io.*;
import javax.swing.*;
import turgen.config.ConfigClass;
import turgen.config.IntegerConfigEntry;
import turgen.playlist.AbstractPlayListItem;
import turgen.tapeimage.*;

public class PlLowerSilesiaTurbo2000Plugin implements Plugin {

    
    private String lastError;
    private PlLowerSilesiaTurbo2000PluginPanel panel;
    private InOutProvider ioProvider;
    private final int[] casChunkName = {116,114,112,57};

    private ConfigClass cfcLowerSilesiaTurbo2000;
    private IntegerConfigEntry cfgPilotLenA;
    private IntegerConfigEntry cfgPilotLenB;
    

    /** New instance */
    public PlLowerSilesiaTurbo2000Plugin() {
           
       
    }

    /** Get plugin name
     * @return Plugin name
     */
    public String getName() {
        return "Lower Silesia Turbo 2000";

    }

    /*************************************************************/
    /**
     * Get generator instructions
     * @param _pl Playlist item
     * @return Instructions
     */
    
    public int[] getInstructions(AbstractPlayListItem _pl) {

        /*Get CsTurboPlaylistItem*/
        PlLowerSilesiaTurbo2000PlaylistItem pl = (PlLowerSilesiaTurbo2000PlaylistItem) _pl;

       try {

            /*Output stream for instructions*/
            InstructionStream is = new InstructionStream();

            /*Nastaveni, pro vsechny mozne konverze je nastaveni stejne,
             *bloky nebo soubory jsou vzdy v Super Turbu o dane rychlosti
             */
            Utils.addSetupInstruction(is, pl.pulses, casChunkName,SignalGenerator.FLAG_POLARITY_10,SignalGenerator.FLAG_ORDER_HL);

            /*Nyni se vybere konverze podle typu v polozce playlistu*/
            switch (pl.conversionType) {

                /*Zde se prevadi cely soubor*/
                case PlLowerSilesiaTurbo2000PlaylistItem.CV_BINARY_AUTOTURBO: {
                    convertAutoTurbo(is, pl);
                    break;
                }
            }

            is.add(SignalGenerator.INSTR_END);

            return is.getInstructions();

        } catch (Exception e) {
            
            e.printStackTrace();
            if (e.getMessage() == null) {
                lastError = "Unable to produce instructions: " + e.toString();
            } else {
                lastError = e.toString();
            }
            return null;

        }
    }

    public String getLastError() {
        return lastError;
    }

    public AbstractPlayListItem createItem() {
        PlLowerSilesiaTurbo2000PlaylistItem dpr = panel.getRecord();
        dpr.pluginClass = Turgen.pluginManager.getPluginClassName(this);
        return dpr;
    }

    public void resetPanel() {
        panel.clear();
    }

    public String validateInput() {
        return panel.verify();
    }

    public JPanel getItemPanel() {
        return panel;
    }

    public void init() {
        panel = new PlLowerSilesiaTurbo2000PluginPanel();
        
        /*Initialize configuration class*/
        cfcLowerSilesiaTurbo2000 = new ConfigClass("LowerSilesiaTurbo2000", "Lower Silesia Turbo 2000");

        /*Initialize configuration entries*/
        cfgPilotLenA = new IntegerConfigEntry("pilotLenA","Header block pilot tone length [pulse]", 3072, 256, 8192);
        cfgPilotLenB = new IntegerConfigEntry("pilotLenB","Data block pilot tone length [pulse]", 3072, 256, 8192);
        
        /*Add entries to the class*/
        cfcLowerSilesiaTurbo2000.addEntry(cfgPilotLenA);
        cfcLowerSilesiaTurbo2000.addEntry(cfgPilotLenB);
        

        /*Register configuration class*/
        Configuration.registerClass(cfcLowerSilesiaTurbo2000);
        
    }

    public void fillPanel(AbstractPlayListItem plr) {
        PlLowerSilesiaTurbo2000PlaylistItem dpr = (PlLowerSilesiaTurbo2000PlaylistItem) plr;
        panel.fill(dpr);
    }

    public void setInOutProvider(InOutProvider prov) {
        ioProvider = prov;
        panel.setIOProvider(prov);
        
    }

    private void convertAutoTurbo(InstructionStream is, PlLowerSilesiaTurbo2000PlaylistItem cpi) throws Exception {

        int plenA = cfgPilotLenA.getAsInt();
        int plenB = cfgPilotLenA.getAsInt();
        
        /*DATA*/
        int[] fileData = readFileData(cpi.inFile, 0, Integer.MAX_VALUE);
        
        int hdrLoad = 0;
        int hdrRun = 0;
        int hdrLength = fileData.length;
        int hdrType = 4;

        /*Header*/
        is.add(SignalGenerator.INSTR_PILOTTONE);
        is.add(plenA);
        is.add(SignalGenerator.INSTR_SYNC);

        is.add(SignalGenerator.INSTR_DATA);
        is.add(19);

        int[] hdr = Utils.createT2KHeader(cpi.fname, hdrType, hdrLoad, hdrLength, hdrRun);
        is.add(hdr);
        
        /*Safety pulse + safety pause*/
        is.add(SignalGenerator.INSTR_NARROW);
        is.add(SignalGenerator.INSTR_BLOCKSEP);
        
        /*Main block*/
        is.add(SignalGenerator.INSTR_PILOTTONE);
        is.add(plenB);
        is.add(SignalGenerator.INSTR_SYNC);

        is.add(SignalGenerator.INSTR_DATA);
        is.add(fileData.length + 2);

        int chsum = 255;
        is.add(chsum);
        for (int i = 0; i < fileData.length; i++) {
            chsum ^= fileData[i];
        }
        is.add(fileData);
        is.add(chsum);
        /*Safety pulse + safety pause*/
        is.add(SignalGenerator.INSTR_NARROW);
        is.add(SignalGenerator.INSTR_BLOCKSEP);

    }

    
    private int[] readFileData(String file, int skipBytes, int maxLen) throws Exception {

        byte[] b;
        RandomAccessFile raf = new RandomAccessFile(file, "r");

        int flen = (int) raf.length();
        if (flen > maxLen) {
            raf.close();
            throw new FileTooLargeException("File exceeds " + Integer.toString(maxLen) + " bytes");
        }

        b = new byte[flen - skipBytes];
        raf.skipBytes(skipBytes);
        raf.readFully(b);
        raf.close();


        /*Konverze na int*/
        int[] fileData = new int[b.length];
        for (int i = 0; i < b.length; i++) {
            fileData[i] = b[i] < 0 ? b[i] + 256 : b[i];
        }

        return fileData;
    }

    public String[] getSupportedTapeImageChunks() {
        return new String[] {"trp9"};
    }


    public void addInstructionsForChunk(InstructionStream is, TapeImageChunk chunk) throws Exception {

        /*trXX chunk*/
        if (chunk instanceof TurboChunk) {
            Utils.addSetupInstruction(is, Turgen.pulseCache.getPulsesForPlugin(this)[0], this.casChunkName,SignalGenerator.FLAG_POLARITY_10,SignalGenerator.FLAG_ORDER_HL);
            return;
        }

        /*Only data chunk accepted*/
        if (!(chunk instanceof DataChunk)) return;
        DataChunk dataChunk = (DataChunk)chunk;

        /*Zavadeci ton a data bloku*/
        is.add(SignalGenerator.INSTR_PILOTTONE);
        is.add(dataChunk.getAux());

        /*Synchropulz*/
        is.add(SignalGenerator.INSTR_SYNC);

        /*Data bloku*/
        is.add(SignalGenerator.INSTR_DATA);
        is.add(dataChunk.getLength());

        int[] dta = dataChunk.getData();

        
        is.add(dta);
        

        /*Safety pulse + safety pause*/
        is.add(SignalGenerator.INSTR_NARROW);
        is.add(SignalGenerator.INSTR_BLOCKSEP);
        
    }

    public BinaryConvertor[] getBinaryConvertors() {
        BinaryConvertor[] bc = new BinaryConvertor[1];
        PlLowerSilesiaTurbo2000AutoTurboBinaryConvertor abc = new PlLowerSilesiaTurbo2000AutoTurboBinaryConvertor(this);
        bc[0]=abc;
        return bc;
    }

    public String getCountry() {
        return "PL";
    }

    public boolean isUsingAlternateFileChooser() {
        return false;
    }


}
